package net.wanji.utc.service.staticinfo.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.LocalDateTimeUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.*;
import net.wanji.databus.po.CrossSchemeRings;
import net.wanji.databus.po.SchemeOptLogPO;
import net.wanji.databus.vo.CrossIdVO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.feign.service.common.FeignCommon;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.UtcFeignClientCache;
import net.wanji.utc.common.Result;
import net.wanji.utc.service.control.impl.ControlCommandStrategyServiceImpl;
import net.wanji.utc.service.staticinfo.WanJiCommonStaticInfoService;
import net.wanji.utc.util.HttpRestUtil;
import net.wanji.utc.util.RedisUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/01/09 9:21
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class WanJiCommonStaticInfoServiceImpl implements WanJiCommonStaticInfoService {

    private final HttpRestUtil httpRestUtil;
    @Value("url")
    private String rpcUri;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final CrossInfoCache crossInfoCache;
    private final UtcFeignClientCache utcFeignClientCache;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final CrossSchemeStageOptLogMapper crossSchemeStageOptLogMapper;
    private final ControlCommandStrategyServiceImpl controlCommandStrategyService;
    private final RedisUtil redisUtil;


    @Override
    public void schemePhaseLights(String crossId, String signalId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        CrossIdVO crossIdVO = new CrossIdVO();
        crossIdVO.setCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).schemePhaseLights(crossIdVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("获取方案数据-方案信息、相位信息、灯组信息远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        ObjectMapper objectMapper = JacksonUtils.getInstance();
        SchemePhaseLightsVO content = objectMapper.convertValue(jsonViewObject.getContent(), SchemePhaseLightsVO.class);
        List<CrossSchemePO> crossSchemeList = content.getCrossSchemeList();
        crossSchemeList.forEach(crossSchemePO -> {
            crossSchemeMapper.deleteOne(crossId, crossSchemePO.getSchemeNo());
            crossSchemeMapper.insertOne(crossSchemePO);
        });
        List<CrossPhasePO> crossPhaseList = content.getCrossPhaseList();
        crossPhaseMapper.deleteBatch(crossId, null);
        crossPhaseList.forEach(crossPhasePO -> {
            CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, crossPhasePO.getPlanId());
            Integer id;
            if (Objects.isNull(crossSchemePO)) {
                id = -1;
            } else {
                id = crossSchemePO.getId();
            }
            crossPhasePO.setPlanId(id);
            crossPhaseMapper.insertOne(crossPhasePO);
        });

        List<CrossLightsPO> crossLightsList = content.getCrossLightsList();
        if (!CollectionUtils.isEmpty(crossLightsList)) {
            crossLightsMapper.deleteByCrossId(crossId);
            crossLightsList.forEach(crossLightsPO -> {
                crossLightsMapper.insertOne(crossLightsPO);
            });
        }
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = content.getCrossPhaseLightsPOList();
        if (!CollectionUtils.isEmpty(crossPhaseLightsPOList)) {
            crossPhaseLightsMapper.deleteByCrossId(crossPhaseLightsPOList.get(0).getCrossId());
            for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
                String currentCrossId = crossPhaseLightsPO.getCrossId();
                Integer lightsId = crossPhaseLightsPO.getLightsId();
                Integer phaseId = crossPhaseLightsPO.getPhaseId();
                List<Integer> list = crossPhaseMapper.selectIdsByPhaseNo(currentCrossId, phaseId);
                if (!CollectionUtils.isEmpty(list)) {
                    crossPhaseLightsPO.setPhaseId(list.get(0));
                }
                CrossLightsPO crossLightsPO = new CrossLightsPO();
                crossLightsPO.setCrossId(currentCrossId);
                crossLightsPO.setLightsNo(String.valueOf(lightsId));
                CrossLightsPO result = crossLightsMapper.selectEntity(crossLightsPO);
                crossPhaseLightsPO.setLightsId(result.getId());
            }
            crossPhaseLightsMapper.insertBatch(crossPhaseLightsPOList);
        }
    }

    @Override
    public void planSection(String crossId, String signalId, Integer planNo) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        PlanSectionDTO planSectionDTO = new PlanSectionDTO();
        planSectionDTO.setCrossId(crossId);
        planSectionDTO.setPlanNo(planNo);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).planSection(planSectionDTO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("获取计划数据-计划信息、时段信息远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        ObjectMapper mapper = JacksonUtils.getInstance();
        List<PlanSectionVO> content = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<PlanSectionVO>>() {
        });
        if (!CollectionUtils.isEmpty(content)) {
            crossSectionMapper.deleteOne(crossId, null, null);
            List<CrossPlanPO> crossPlanPOList = content.get(0).getCrossPlanPOList();
            crossPlanPOList.forEach(crossPlanPO -> {
                crossPlanMapper.deleteOne(crossId, Integer.parseInt(crossPlanPO.getPlanNo()));
                crossPlanMapper.insertOne(crossPlanPO);
            });
            for (PlanSectionVO planSectionVO : content) {
                List<CrossSectionPO> crossSectionPOList = planSectionVO.getCrossSectionPOList();
                crossSectionPOList.forEach(crossSectionPO -> {
//                    crossSectionMapper.deleteOne(crossId, Integer.parseInt(crossSectionPO.getSectionNo()), null);
                    Integer id = crossPlanMapper.selectIdByNo(crossSectionPO.getCrossId(), crossSectionPO.getPlanId());
                    crossSectionPO.setPlanId(id);
                    CrossSchemePO baseCrossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, crossSectionPO.getSchemeId());
                    if (Objects.nonNull(baseCrossSchemePO)) {
                        crossSectionPO.setSchemeId(baseCrossSchemePO.getId());
//                        crossSectionPO.setSchemeId(crossSectionPO.getSchemeId());
                        crossSectionMapper.insertOne(crossSectionPO);
                    }
                });
            }
        }
    }

    @Override
    public Result<List<CrossSchedulesPO>> schedules(String crossId, String signalId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        CrossSchedulesDTO crossSchedulesDTO = new CrossSchedulesDTO();
        crossSchedulesDTO.setManufacturerCode(manufacturerIdCode);
        crossSchedulesDTO.setCrossIdList(Arrays.asList(crossId));
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).crossSchedules(crossSchedulesDTO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("获取时间表数据远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        ObjectMapper mapper = JacksonUtils.getInstance();
        List<CrossSchedulesPO> content = mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<CrossSchedulesPO>>() {
        });

        if (Objects.nonNull(content)) {
            crossSchedulesMapper.deleteByCrossId(crossId);
            if (!CollectionUtils.isEmpty(content)) {
                for (CrossSchedulesPO crossSchedulesPO : content) {
                    Integer planId = crossSchedulesPO.getPlanId();
                    Integer id = crossPlanMapper.selectIdByNo(crossId, planId);
                    crossSchedulesPO.setPlanId(id);
                    if (Objects.nonNull(id)) {
                        crossSchedulesMapper.insertBatch(Collections.singletonList(crossSchedulesPO));
                    }
                }
            }
        }
        return Result.success(content);
    }

    @Override
    public JsonViewObject signalTime(String crossId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).signalTime(crossId);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("获取信号机时间远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject crossSchemeNo(String crossId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        FeignCommon utcFeignClientService = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode);
        JsonViewObject jsonViewObject = utcFeignClientService.getSchemeNo(crossId);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("获取当前运行的方案号远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject crossSchemeRings(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        String now = LocalDateTimeUtil.formatNow(LocalDateTimeUtil.TIMEFORMATTER);

        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossSchemeRingsDTO.getCrossId());
        JsonViewObject jsonViewObject = null;
        String crossId = crossSchemeRingsDTO.getCrossId();
        if ("HISENSE".equals(manufacturerIdCode)) {
            FeignCommon utcFeignClientService = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode);
            jsonViewObject = utcFeignClientService.crossSchemeRings(crossSchemeRingsDTO);
            log.info("方案环图请求,crossId:{},result:{}",crossId,jsonViewObject);
            Integer code = jsonViewObject.getCode();
            if (code != 200){
                JSONObject jsonObject = JSONObject.parseObject(JSON.toJSONString(crossSchemeRingsDTO));
                //插入命令操作日志
//                controlCommandStrategyService.insertCommandLog(now,jsonViewObject, jsonObject,10);
            }
        } else {
            CrossSchemeRings crossSchemeRings = getCrossSchemeRings(crossSchemeRingsDTO);
            if (ObjectUtil.isNotEmpty(crossSchemeRings)) {
                jsonViewObject = JsonViewObject.newInstance().success(crossSchemeRings);
            } else {
                jsonViewObject = JsonViewObject.newInstance().fail("该方案信息未同步");
            }
        }
        /*if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("方案环图请求远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }*/
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            CrossSchemeRings crossSchemeRings = mapper.convertValue(jsonViewObject.getContent(), CrossSchemeRings.class); // CrossSchemeRings
            redisUtil.setHash("utc_scheme_offset", crossId, mapper.writeValueAsString(crossSchemeRings));
        } catch (Exception e) {
            log.error("方案换图请求存储redis失败：{}", e);
//            throw new RuntimeException(e);
        }

        return jsonViewObject;
    }

    private CrossSchemeRings getCrossSchemeRings(CrossSchemeRingsDTO crossSchemeRingsDTO) {
        CrossSchemeRings crossSchemeRings = null;
        String crossId = crossSchemeRingsDTO.getCrossId();
        QueryByCrossIdAndTimeDTO queryByCrossIdAndTimeDTO = new QueryByCrossIdAndTimeDTO();
        queryByCrossIdAndTimeDTO.setCrossId(crossId);
        queryByCrossIdAndTimeDTO.setDatetime(new Date());
        //获取当前运行的方案号
        BaseCrossSchemePO baseCrossSchemePO = getBaseCrossSchemePO(queryByCrossIdAndTimeDTO);
        if (ObjectUtil.isEmpty(baseCrossSchemePO)) {
            return crossSchemeRings;
        }
        Integer schemeId = baseCrossSchemePO.getId();
        //通过方案查询相位信息
        List<CrossPhasePO> phasePOList = crossPhaseMapper.selectByCrossIdAndPlanId(crossId, String.valueOf(schemeId));
        if (ObjectUtil.isNotEmpty(phasePOList)) {
            List<String> dirList = new ArrayList<>();
            //更新方向转向信息
            for (CrossPhasePO crossPhasePO : phasePOList) {
                List<CrossLightsPO> crossLightsPOS = crossLightsMapper.selectByphaseId(crossPhasePO.getId());
                String[] array = crossLightsPOS.stream().map(x -> x.getDir() + "_" + x.getTurn()).toArray(String[] :: new);
                String join = StringUtils.join(array, ",");
                dirList.add(join);
            }

            crossSchemeRings = new CrossSchemeRings();
            List<Integer> greenList = phasePOList.stream().map(x -> x.getGreenTime()).collect(Collectors.toList());
            List<Integer> redList = phasePOList.stream().map(x -> x.getRedTime()).collect(Collectors.toList());
            List<Integer> yellowList = phasePOList.stream().map(x -> x.getYellowTime()).collect(Collectors.toList());
            List<Integer> maxGreenList = phasePOList.stream().map(x -> x.getMaxGreenTime()).collect(Collectors.toList());
            List<Integer> minGreenList = phasePOList.stream().map(x -> x.getMinGreenTime()).collect(Collectors.toList());
            crossSchemeRings.setSpot(crossId);
            crossSchemeRings.setPattern(baseCrossSchemePO.getSchemeNo());
            crossSchemeRings.setCycleLen(String.valueOf(baseCrossSchemePO.getCycle()));
            crossSchemeRings.setRingCount("1");
            crossSchemeRings.setOffset(String.valueOf(baseCrossSchemePO.getOffset()));
            JSONObject jsonObj = new JSONObject();
            String[] phaseArray = phasePOList.stream().map(x -> String.valueOf(x.getSort())).toArray(String[]::new);
            jsonObj.put("Cycle1",StringUtils.join(phaseArray," "));
            crossSchemeRings.setCycleList(jsonObj);
            List<CrossSchemeRings.Phase> phaseList = new ArrayList<>();
            CrossSchemeRings.Phase phase = new CrossSchemeRings.Phase();
            phase.setRedTime(changeTimeList(redList,16));
            phase.setYellowTime(changeTimeList(yellowList,16));
            phase.setGreenTime(changeTimeList(greenList,16));
            phase.setDirection(changePhase(dirList,16));
            phase.setMinGTime(changeTimeList(minGreenList,16));
            phase.setMaxGTime(changeTimeList(maxGreenList,16));
            phase.setChannelDim(changeTimeList(null,16));
            phase.setConcurrent(changeTimeList(null,16));
            phaseList.add(phase);
            crossSchemeRings.setPhaseList(phaseList);
            List<CrossSchemeRings.OverLapPhase> overLapPhaseList = new ArrayList<>();
            CrossSchemeRings.OverLapPhase overLapPhase = new CrossSchemeRings.OverLapPhase();
            overLapPhase.setDirection(changeTimeList(null,16));
            overLapPhase.setIncludedPhases(changeTimeList(null,16));
            overLapPhase.setModifierPhases(changeTimeList(null,16));
            overLapPhaseList.add(overLapPhase);
            crossSchemeRings.setOverLapPhaseList(overLapPhaseList);
        }

        return crossSchemeRings;

    }

    /**
     * @return java.lang.String
     * @Description 包装数据，将list数据转换为String ，中间用空格隔开
     * @Param [list, length]
     **/
    private static String changeTimeList(List<Integer> list, int length) {
        if (ObjectUtil.isEmpty(list)) {
            list = new ArrayList<>();
        }
        int size = list.size();
        List<Integer> addList = new ArrayList<>();
        for (int i = 0; i < length - size; i++){
            addList.add(0);
        }
        list.addAll(addList);
        String[] array = list.stream().map(String::valueOf).toArray(String[] :: new);
        String join = StringUtils.join(array, " ");
        return join;
    }

    /**
     * @return java.lang.String
     * @Description 包装数据，将list数据转换为String ，中间用空格隔开
     * @Param [list, length]
     **/
    private static String changePhase(List<String> list, int length) {
        if (ObjectUtil.isEmpty(list)) {
            list = new ArrayList<>();
        }
        int size = list.size();
        List<String> addList = new ArrayList<>();
        for (int i = 0; i < length - size; i++){
            addList.add("0");
        }
        list.addAll(addList);
        String[] array = list.stream().map(String::valueOf).toArray(String[] :: new);
        String join = StringUtils.join(array, " ");
        return join;
    }

    @Override
    public JsonViewObject schemeOptLog(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        LambdaQueryWrapper<CrossSchemeStageOptLogPO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CrossSchemeStageOptLogPO::getCrossId, crossSchemeRingsDTO.getCrossId())
                .eq(CrossSchemeStageOptLogPO::getSchemeId, crossSchemeRingsDTO.getPattern())
                .inSql(CrossSchemeStageOptLogPO::getBatchTime, "select max(batch_time) from t_cross_scheme_opt_log where cross_id = '" + crossSchemeRingsDTO.getCrossId() + "' and scheme_id = " + crossSchemeRingsDTO.getPattern());
        List<CrossSchemeStageOptLogPO> crossSchemeStageOptLogPOS = crossSchemeStageOptLogMapper.selectList(queryWrapper);
        return JsonViewObject.newInstance().success(crossSchemeStageOptLogPOS);
    }

    @Override
    public JsonViewObject selectSchemeByParams(QueryByCrossIdAndTimeDTO queryByCrossIdAndTimeDTO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(queryByCrossIdAndTimeDTO.getCrossId());
        JsonViewObject jsonViewObject = null;
        if ("HISENSE".equals(manufacturerIdCode)) {
            FeignCommon utcFeignClientService = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode);
            jsonViewObject = utcFeignClientService.selectSchemeByParams(queryByCrossIdAndTimeDTO);
        } else {
            BaseCrossSchemePO baseCrossSchemePO = getBaseCrossSchemePO(queryByCrossIdAndTimeDTO);
            if (ObjectUtil.isNotEmpty(baseCrossSchemePO)) {
                jsonViewObject = JsonViewObject.newInstance().success(baseCrossSchemePO);
            } else {
                jsonViewObject = JsonViewObject.newInstance().fail();
            }
        }

        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            throw new Exception("根据时间和路口号查询方案号远程服务调用异常，异常信息" + jsonViewObject.getMessage());
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject selectSchemeOptLogByParams(CrossSchemeQueryDTO crossSchemeQueryDT) throws Exception {
        String crossId = crossSchemeQueryDT.getCrossId();
        String pattern = crossSchemeQueryDT.getPattern();
        Date datetime = crossSchemeQueryDT.getDatetime();
        long time = datetime.getTime();
        SchemeOptLogPO schemeOptLogPO = new SchemeOptLogPO();
        List<CrossSchemeStageOptLogPO> beforeTimeList = crossSchemeStageOptLogMapper.getBeforeTimeList(crossId, pattern, time);
        List<CrossSchemeStageOptLogPO> afterTimeList = crossSchemeStageOptLogMapper.getAfterTimeList(crossId, pattern, time);
        schemeOptLogPO.setBeforeTime(beforeTimeList);
        schemeOptLogPO.setAfterTime(afterTimeList);
        return JsonViewObject.newInstance().success(schemeOptLogPO);
    }

    /**
     * @return net.wanji.databus.dao.entity.BaseCrossSchemePO
     * @Description 获取方案号
     * @Param [queryByCrossIdAndTimeDTO]
     **/

    private BaseCrossSchemePO getBaseCrossSchemePO(QueryByCrossIdAndTimeDTO queryByCrossIdAndTimeDTO) {
        Date datetime = queryByCrossIdAndTimeDTO.getDatetime();
        //java.util.date转LocalDate
        LocalDate localDate = datetime.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        int week = localDate.getDayOfWeek().getValue();
        if (week == 7) {
            //表中周日用0表示
            week = 0;
        }
        // 创建一个 SimpleDateFormat 对象，并指定日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        // 将 Date 对象格式化为字符串
        String formattedDate = sdf.format(datetime);
        return baseCrossSchemeMapper.selectSchemeByParams(queryByCrossIdAndTimeDTO.getCrossId(), formattedDate, week);
    }

}
